BoxFusedMultipartWrite

Type Alias BoxFusedMultipartWrite 

pub type BoxFusedMultipartWrite<'a, Part, R, T, E> = Pin<Box<dyn FusedMultipartWrite<Part, Recv = R, Output = T, Error = E> + Send + 'a>>;
An owned, dynamically typed FusedMultipartWrite for use in cases where it is not possible or desirable to statically type it.

This is also handy to aid in type inference. Since a FusedMultipartWrite is generic over the type of value being written, often it is not possible to infer the types of Part, R, T, and/or E.

Erasing the type with BoxFusedMultipartWrite picks the implementation and can resolve a string of type inference failures.

Aliased Type

pub struct BoxFusedMultipartWrite<'a, Part, R, T, E> { /* private fields */ }