pub type BoxMultipartWrite<'a, Part, R, T, E> = Pin<Box<dyn MultipartWrite<Part, Recv = R, Output = T, Error = E> + Send + 'a>>;Expand description
An owned, dynamically typed MultipartWrite for use in cases where it is
not possible or desirable to statically type it.
This is also handy to aid in type inference. Since a MultipartWrite is
generic over the type of value being written, often it is not possible to
infer the types of Part, R, T, and/or E.
Erasing the type with BoxMultipartWrite picks the implementation and can
resolve a string of type inference failures.
Aliased Type
pub struct BoxMultipartWrite<'a, Part, R, T, E> { /* private fields */ }