Module write

Module write 

MultipartWrite combinators.

This module contains the trait MultipartWriteExt, which provides adapters for chaining and composing writers.

Structs

Buffered
MultipartWrite for buffered.
Complete
Future for complete.
Extend
MultipartWrite for extend.
Fanout
MultipartWrite for fanout.
Feed
Future for feed.
FilterMapPart
MultipartWrite for filter_map_part.
FilterPart
MultipartWrite for filter_part.
Flush
Future for flush.
FoldSent
MultipartWrite for fold_sent.
ForEachRecv
MultipartWrite for for_each_recv.
Fuse
MultipartWrite for fuse.
Lift
MultipartWrite for lift.
MapErr
MultipartWrite for map_err.
MapOk
MultipartWrite for map_ok.
MapSent
MultipartWrite for map_sent.
ReadyPart
MultipartWrite for ready_part.
SendFlush
Future for send_flush.
Then
MultipartWrite for then.

Traits

MultipartWriteExt
An extension trait for MultipartWrite providing a variety of convenient combinator functions.

Functions

extend
Returns a value that becomes a MultipartWrite over any A where T: std::iter::Extend<A>.
extend_default
extend but starts with the default value of T.