from_extend

Function from_extend 

pub fn from_extend<A, T: Unpin + Default + Extend<A>>() -> FromExtend<A, T>
Function that creates a MultipartWrite from a type that implements std::iter::Extend<A> + Default for some A.

A becomes the part type in the multipart writer. This writer is always ready to receive a value.