pub struct Fuse<Wr, Part, F> { /* private fields */ }Expand description
MultipartWrite for fuse.
Implementations
Sourceimpl<Wr, Part, F> Fuse<Wr, Part, F>
impl<Wr, Part, F> Fuse<Wr, Part, F>
Sourcepub fn into_inner(self) -> Wr
pub fn into_inner(self) -> Wr
Consumes Fuse, returning the underlying writer.
Sourcepub fn get_mut(&mut self) -> &mut Wr
pub fn get_mut(&mut self) -> &mut Wr
Acquires a mutable reference to the underlying writer.
It is inadvisable to directly write to the underlying writer.
Sourcepub fn get_pin_mut(self: Pin<&mut Self>) -> Pin<&mut Wr>
pub fn get_pin_mut(self: Pin<&mut Self>) -> Pin<&mut Wr>
Acquires a pinned mutable reference to the underlying writer.
It is inadvisable to directly write to the underlying writer.