Module write

Module write 

MultipartWrite combinators.

This module contains the trait MultipartWriteExt, which provides adapters for chaining and composing MultipartWriters.

Structs

AndThen
MultipartWrite for and_then.
Buffered
MultipartWrite for the buffered method.
Complete
Future for complete.
Fanout
MultipartWrite for fanout.
Feed
Future for feed.
FilterMapPart
MultipartWrite for filter_map_part.
FilterPart
MultipartWrite for filter_part.
Flush
Future for flush.
FoldSent
MultipartWrite for fold_sent.
FromExtend
MultipartWrite for the from_extend function.
Fuse
MultipartWrite for fuse.
Lift
MultipartWrite for lift.
MapErr
MultipartWrite for map_err.
MapOk
MultipartWrite for map_ok.
MapSent
MultipartWrite for map_sent.
ReadyPart
MultipartWrite for ready_part.
Resolve
Writer returned by resolve.
SendFlush
Future for send_flush.

Traits

MultipartWriteExt
An extension trait for MultipartWrite providing a variety of convenient combinator functions.

Functions

from_extend
Function that constructs a MultipartWrite from any value that has a default and implements std::iter::Extend.
resolve
Create a MultipartWrite from a future that resolves to one.